@@ -15,6 +15,7 @@ use strict;
 use warnings;
 use LaTeXML::Util::Pathname;
 use LaTeXML::Common::XML;
+use LaTeXML::Common::Error;
 use LaTeXML::Post;
 use base qw(LaTeXML::Post::Processor);
 
@@ -27,42 +28,46 @@ use base qw(LaTeXML::Post::Processor);
 #         CSS   is a '|' separated list of paths
 #         ICON  a favicon
 #         resource_directory a directory under top-level to put resources (css, js, etc)
+#         searchpaths : list of paths to search
 sub new {
   my ($class, %options) = @_;
   my $self       = $class->SUPER::new(%options);
   my $stylesheet = $options{stylesheet};
   Error('expected', 'stylesheet', undef, "No stylesheet specified!") unless $stylesheet;
   if (!ref $stylesheet) {
+    $$self{name} .= '[using ' . $stylesheet . ']';
     my $pathname = pathname_find($stylesheet,
-      types => ['xsl'], installation_subdir => 'resources/XSLT');
+      types => ['xsl'], installation_subdir => 'resources/XSLT',
+      paths => $$self{searchpaths} || ['.']);
     Error('missing-file', $stylesheet, undef, "No stylesheet '$stylesheet' found!")
       unless $pathname && -f $pathname;
     $stylesheet = $pathname; }
-  $stylesheet = LaTeXML::Common::XML::XSLT->new($stylesheet);
+  $stylesheet = $stylesheet && LaTeXML::Common::XML::XSLT->new($stylesheet);
   if ((!ref $stylesheet) || !($stylesheet->can('transform'))) {
     Error('expected', 'stylesheet', undef, "Stylesheet '$stylesheet' is not a usable stylesheet!"); }
+  XML::LibXSLT->max_depth(1000);
   $$self{stylesheet} = $stylesheet;
   my %params = ();
-  %params = %{ $options{parameters} } if $options{parameters};
+  %params                    = %{ $options{parameters} } if $options{parameters};
   $$self{parameters}         = {%params};
   $$self{noresources}        = $options{noresources};
-  $$self{resource_directory} = $options{resource_directory};    # ???
+  $$self{resource_directory} = $options{resource_directory};                        # ???
   return $self; }
 
 sub process {
   my ($self, $doc, $root) = @_;
+  return unless $$self{stylesheet};
   # # Set up the Stylesheet parameters; making pathname parameters relative to document
   my %params = %{ $$self{parameters} };
-
   # Deal with any resources embedded within the document
   if (my @resnodes = $doc->findnodes('//ltx:resource[@src]')) {
     if ($$self{noresources}) {
       $doc->removeNodes(@resnodes); }
     else {
       foreach my $node (@resnodes) {
-        my $src = $node->getAttribute('src');
+        my $src  = $node->getAttribute('src');
         my $path = $self->copyResource($doc, $src, $node->getAttribute('type'));
-        $node->setAttribute(src => $path) unless $path eq $src; } } }
+        $node->setAttribute(src => pathname_to_url($path)) unless $path eq $src; } } }
   if (my $css = $params{CSS}) {
     $params{CSS} = '"' . join('|', map { $self->copyResource($doc, $_, 'text/css') } @$css) . '"'; }
   if (my $js = $params{JAVASCRIPT}) {
@@ -82,9 +87,9 @@ my $RESOURCE_INFO = {    # [CONSTANT]
 # (in case it has been adjusted to be local to the destination document)
 sub copyResource {
   my ($self, $doc, $reqsrc, $type) = @_;
-  my $ext    = $type && $$RESOURCE_INFO{$type}{extension};
-  my $resdir = $type && $$RESOURCE_INFO{$type}{subdir};
-  my @searchpaths => $doc->getSearchPaths;
+  my $ext         = $type && $$RESOURCE_INFO{$type}{extension};
+  my $resdir      = $type && $$RESOURCE_INFO{$type}{subdir};
+  my @searchpaths = ($doc->getSearchPaths, ($$self{searchpaths} ? @{ $$self{searchpaths} } : ()));
   # If the $reqsrc is a URL, no need to copy the resource, or modify the path to it.
   if (pathname_is_url($reqsrc)) {
     return $reqsrc; }
@@ -92,32 +97,31 @@ sub copyResource {
   elsif (my $path = pathname_find($reqsrc, ($ext ? (types => [$ext]) : ()),
       paths => [@searchpaths],
       ($resdir ? (installation_subdir => $resdir) : ()))) {
-    # Make an attempt to preserve the relative path to the requested resource
+    my $dest;
+    my ($dir, $name, $ex) = pathname_split($path);
+    # If a resource directory has been specified,
+    if (my $rd = $$self{resource_directory}) {
+      # Put resources in resource_directory, relative to site_directory
+      $dest = pathname_absolute(pathname_make(dir => $rd, name => $name, type => $ex),
+        $doc->getSiteDirectory); }
+    # Otherise, make an attempt to preserve the relative path to the requested resource
     # Ie. same path from dest doc to copied resource, as from original doc to source resource.
     # Get the path relative to user's source, and simulate that path in the destination
-    my $relpath = pathname_relative($reqsrc, $doc->getSourceDirectory);
-    if (my $rd = $$self{resource_directory}) {
-      $relpath = pathname_concat($rd, $relpath); }
-    my $dest = pathname_absolute($relpath, $doc->getSiteDirectory);
-    # Now IFF that is a valid relative path WITHIN the site directory, we'll use it.
-    # Otherwise, we'll place it in a resource directory, or at toplevel in the destination.
-    if (!pathname_is_contained($dest, $doc->getSiteDirectory)) {
-      # $resourcedir can be relative (interpreted relative to destination doc)
-      # or absolute, HOPEFULLY, within the site directory
-      my ($dir, $name, $ex) = pathname_split($dest);
-      $relpath = pathname_make(dir => $$self{resource_directory} || '', name => $name, type => $ex);
-      $relpath = pathname_relative($relpath, $doc->getSourceDirectory)
-        if pathname_is_absolute($relpath);
-      $dest = pathname_absolute($relpath, $doc->getSiteDirectory); }
-
+    else {
+      my $relpath = pathname_relative($reqsrc, $doc->getSourceDirectory);
+      $dest = pathname_absolute($relpath, $doc->getSiteDirectory);
+      # Now IFF that is a valid relative path WITHIN the site directory, we'll use it.
+      # Otherwise, we'll place at toplevel in the destination.
+      if (!pathname_is_contained($dest, $doc->getSiteDirectory)) {
+        $dest = pathname_make(dir => $doc->getSiteDirectory, name => $name, type => $ex); } }
     # Now, copy (unless in same place! happens a lot during testing!!!!)
     pathname_copy($path, $dest) unless $path eq $dest;
     # and return the relative path from the dest doc to the resource
     return pathname_relative($dest, $doc->getDestinationDirectory); }
   else {
-    warn "Couldn't find resource file $reqsrc in paths " . join(',', @searchpaths) . "\n";
+    Warn('missing_file', $reqsrc, $self,
+      "Couldn't find resource file $reqsrc in paths " . join(',', @searchpaths));
     return $reqsrc; } }
 
 # ================================================================================
 1;
-
